semmed = {
    "AnatomicalEntity": {
        "affected_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene",
            "CellularComponent"
        ],
        "affects": [
            "Disease",
            "BiologicalProcess"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "disrupted_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene"
        ],
        "manifestation_of": [
            "BiologicalProcess"
        ],
        "manifested_by": [
            "Disease"
        ],
        "negatively_regulated_by": [
            "ChemicalSubstance"
        ],
        "negatively_regulates": [
            "ChemicalSubstance",
            "Gene"
        ],
        "physically_interacts_with": [
            "ChemicalSubstance",
            "Gene"
        ],
        "positively_regulated_by": [
            "ChemicalSubstance",
            "Gene"
        ],
        "positively_regulates": [
            "ChemicalSubstance",
            "Gene"
        ],
        "preceded_by": [
            "Disease",
            "BiologicalProcess"
        ],
        "precedes": [
            "Disease",
            "BiologicalProcess"
        ],
        "predisposes": [
            "Disease"
        ],
        "prevents": [
            "Disease"
        ],
        "related_to": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ]
    },
    "BiologicalProcess": {
        "affected_by": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "affects": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "caused_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "disrupted_by": [
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "disrupts": [
            "AnatomicalEntity",
            "Cell",
            "BiologicalProcess",
            "Disease",
            "Gene",
            "CellularComponent"
        ],
        "negatively_regulated_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "negatively_regulates": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene"
        ],
        "physically_interacts_with": [
            "ChemicalSubstance",
            "Disease",
            "BiologicalProcess",
            "Gene"
        ],
        "positively_regulated_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene"
        ],
        "positively_regulates": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Gene"
        ],
        "prevents": [
            "Disease"
        ],
        "related_to": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "treats": [
            "Disease"
        ]
    },
    "Cell": {
        "affected_by": [
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "affects": [
            "Disease",
            "Cell"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "coexists_with": [
            "Cell",
            "ChemicalSubstance",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "derives_from": [
            "ChemicalSubstance"
        ],
        "disrupted_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "disrupts": [
            "BiologicalProcess"
        ],
        "has_part": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "located_in": [
            "CellularComponent",
            "AnatomicalEntity",
            "Cell"
        ],
        "location_of": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "manifestation_of": [
            "Disease",
            "BiologicalProcess"
        ],
        "manifested_by": [
            "Disease"
        ],
        "negatively_regulated_by": [
            "ChemicalSubstance"
        ],
        "negatively_regulates": [
            "ChemicalSubstance",
            "Gene"
        ],
        "part_of": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene",
            "CellularComponent"
        ],
        "physically_interacts_with": [
            "ChemicalSubstance",
            "Gene"
        ],
        "positively_regulated_by": [
            "ChemicalSubstance",
            "Gene"
        ],
        "positively_regulates": [
            "ChemicalSubstance",
            "Gene"
        ],
        "predisposes": [
            "Disease"
        ],
        "produced_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene",
            "CellularComponent"
        ],
        "produces": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "related_to": [
            "Disease",
            "Gene"
        ],
        "subclass_of": [
            "Disease",
            "BiologicalProcess"
        ],
        "treats": [
            "Disease"
        ]
    },
    "CellularComponent": {
        "affected_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "affects": [
            "AnatomicalEntity",
            "Cell",
            "BiologicalProcess",
            "Disease",
            "Gene",
            "CellularComponent"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "disrupted_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "disrupts": [
            "Disease",
            "BiologicalProcess",
            "Cell"
        ],
        "located_in": [
            "CellularComponent",
            "AnatomicalEntity",
            "Cell"
        ],
        "negatively_regulated_by": [
            "CellularComponent",
            "ChemicalSubstance",
            "Gene"
        ],
        "negatively_regulates": [
            "CellularComponent",
            "ChemicalSubstance",
            "Gene"
        ],
        "physically_interacts_with": [
            "CellularComponent",
            "ChemicalSubstance",
            "Disease",
            "Gene"
        ],
        "positively_regulated_by": [
            "CellularComponent",
            "ChemicalSubstance",
            "Gene"
        ],
        "positively_regulates": [
            "CellularComponent",
            "ChemicalSubstance",
            "Gene"
        ],
        "predisposes": [
            "Disease"
        ],
        "related_to": [
            "Disease",
            "AnatomicalEntity",
            "BiologicalProcess"
        ],
        "treats": [
            "Disease"
        ]
    },
    "ChemicalSubstance": {
        "affected_by": [
            "ChemicalSubstance",
            "Disease",
            "BiologicalProcess",
            "Gene"
        ],
        "affects": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "coexists_with": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "disrupted_by": [
            "ChemicalSubstance",
            "Gene"
        ],
        "disrupts": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "negatively_regulated_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "negatively_regulates": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "physically_interacts_with": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "positively_regulated_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "positively_regulates": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "prevents": [
            "Disease"
        ],
        "produced_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "produces": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "PhenotypicFeature",
            "Gene"
        ],
        "related_to": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "treats": [
            "Disease"
        ]
    },
    "Disease": {
        "affected_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "affects": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "caused_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "coexists_with": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "derives_from": [
            "ChemicalSubstance",
            "Gene"
        ],
        "derives_info": [
            "ChemicalSubstance"
        ],
        "disrupted_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "disrupts": [
            "AnatomicalEntity",
            "Cell",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "negatively_regulated_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "negatively_regulates": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene"
        ],
        "physically_interacts_with": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "positively_regulated_by": [
            "ChemicalSubstance",
            "Disease",
            "Gene"
        ],
        "positively_regulates": [
            "ChemicalSubstance",
            "Disease",
            "BiologicalProcess",
            "Gene"
        ],
        "prevented_by": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "prevents": [
            "Disease"
        ],
        "related_to": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "treated_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "treats": [
            "Disease"
        ]
    },
    "Gene": {
        "affected_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene",
            "CellularComponent"
        ],
        "affects": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "disrupted_by": [
            "ChemicalSubstance",
            "Disease",
            "BiologicalProcess",
            "Gene"
        ],
        "disrupts": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "negatively_regulated_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "negatively_regulates": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "physically_interacts_with": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "positively_regulated_by": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "positively_regulates": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene",
            "CellularComponent"
        ],
        "prevents": [
            "Disease"
        ],
        "related_to": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "treats": [
            "Disease"
        ]
    },
    "PhenotypicFeature": {
        "affected_by": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene"
        ],
        "affects": [
            "CellularComponent",
            "Disease",
            "BiologicalProcess",
            "Cell"
        ],
        "causes": [
            "Disease",
            "BiologicalProcess"
        ],
        "disrupted_by": [
            "ChemicalSubstance",
            "Disease",
            "Gene"
        ],
        "disrupts": [
            "CellularComponent",
            "Disease",
            "BiologicalProcess",
            "Cell"
        ],
        "has_part": [
            "ChemicalSubstance",
            "Disease",
            "CellularComponent",
            "Gene"
        ],
        "has_subclass": [
            "Disease",
            "PhenotypicFeature"
        ],
        "located_in": [
            "CellularComponent",
            "AnatomicalEntity",
            "Cell"
        ],
        "manifestation_of": [
            "Disease",
            "BiologicalProcess",
            "Gene"
        ],
        "manifested_by": [
            "Disease"
        ],
        "negatively_regulated_by": [
            "ChemicalSubstance",
            "Gene"
        ],
        "negatively_regulates": [
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene"
        ],
        "part_of": [
            "AnatomicalEntity",
            "Cell",
            "ChemicalSubstance",
            "Disease",
            "CellularComponent"
        ],
        "physically_interacts_with": [
            "ChemicalSubstance",
            "Disease",
            "Gene"
        ],
        "positively_regulated_by": [
            "ChemicalSubstance",
            "Gene"
        ],
        "positively_regulates": [
            "ChemicalSubstance",
            "Gene"
        ],
        "preceded_by": [
            "Disease",
            "BiologicalProcess"
        ],
        "precedes": [
            "BiologicalProcess"
        ],
        "predisposes": [
            "Disease"
        ],
        "prevents": [
            "Disease"
        ],
        "produced_by": [
            "Cell",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene",
            "CellularComponent"
        ],
        "produces": [
            "ChemicalSubstance",
            "Gene"
        ],
        "related_to": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "Gene"
        ],
        "subclass_of": [
            "AnatomicalEntity",
            "ChemicalSubstance",
            "BiologicalProcess",
            "Disease",
            "PhenotypicFeature",
            "Gene"
        ],
        "treats": [
            "Disease"
        ]
    }
}